@@ -306,7 +306,7 @@
     else if (c == N0 || a == 0) {
       ll <- 0
       ul <- uniroot(function(or) {
-        sum(sapply(max(0, a + c - N0):a, dFNCHypergeo, N1, N0, a + c, or)) - dFNCHypergeo(a, N1, N0, a + c, or)/2 - (1 - conf.level)/2
+        sum(sapply(max(0, a + c - N0):a, dFNCHypergeo, N1, N0, a + c, or)) - dFNCHypergeo(a, N1, N0, a + c, or)/2 - (1- conf.level)/2
       }, interval = interval)$root
     }
     else if (a == N1 || c == 0) {
@@ -326,13 +326,13 @@
     c(cfOR.p, ll, ul)
   }
   
-  # dFNCHypergeo <- function (x, m1, m2, n, odds, precision = 1e-07){
-  #    stopifnot(is.numeric(x), is.numeric(m1), is.numeric(m2), 
-  #    is.numeric(n), is.numeric(odds), is.numeric(precision))
-  #    .Call("dFNCHypergeo", as.integer(x), as.integer(m1), as.integer(m2), 
-  #    as.integer(n), as.double(odds), as.double(precision), 
-  #   PACKAGE = "BiasedUrn")
-  # }
+   # dFNCHypergeo <- function(x, m1, m2, n, odds, precision = 1e-07){
+   #    stopifnot(is.numeric(x), is.numeric(m1), is.numeric(m2), 
+   #    is.numeric(n), is.numeric(odds), is.numeric(precision))
+   #    .Call("dFNCHypergeo", as.integer(x), as.integer(m1), as.integer(m2), 
+   #    as.integer(n), as.double(odds), as.double(precision), 
+   #   PACKAGE = "BiasedUrn")
+   # }
   
   # See http://www.stat.ufl.edu/~aa/cda/R/two-sample/R2/index.html
   # See https://stackoverflow.com/questions/4357827/do-while-loop-in-r
@@ -806,23 +806,44 @@
     wOR.u <- .tmp[3]
   }
   
-  
-  # Individual strata odds ratio - Cornfield confidence limits:
-  cfOR.ctype <- "Cornfield"
-  cfOR.p <- c(); cfOR.l <- c(); cfOR.u <- c()
-  if(length(dim(dat)) == 3){
-    for(i in 1:dim(dat)[3]){
-      .tmp <- .funORcfield(dat[,,i], conf.level)
-      cfOR.p <- c(cfOR.p, .tmp[1])
-      cfOR.l <- c(cfOR.l, .tmp[2])
-      cfOR.u <- c(cfOR.u, .tmp[3])
+  # Individual strata odds ratio - Cornfield confidence limits. 
+  # Only calculate Cornfield confidence limits if N < 500; function very slow with large numbers otherwise:
+  if(sum(total) < 500){ 
+    cfOR.ctype <- "Cornfield"
+    cfOR.p <- c(); cfOR.l <- c(); cfOR.u <- c()
+    if(length(dim(dat)) == 3){
+      for(i in 1:dim(dat)[3]){
+        .tmp <- .funORcfield(dat[,,i], conf.level)
+        cfOR.p <- c(cfOR.p, .tmp[1])
+        cfOR.l <- c(cfOR.l, .tmp[2])
+        cfOR.u <- c(cfOR.u, .tmp[3])
+      }
+    }
+    if(length(dim(dat)) == 2){
+      .tmp <- .funORcfield(dat, conf.level)
+      cfOR.p <- .tmp[1]
+      cfOR.l <- .tmp[2]
+      cfOR.u <- .tmp[3]
     }
   }
-  if(length(dim(dat)) == 2){
-    .tmp <- .funORcfield(dat, conf.level)
-    cfOR.p <- .tmp[1]
-    cfOR.l <- .tmp[2]
-    cfOR.u <- .tmp[3]
+  
+  if(sum(total) >= 500){ 
+    cfOR.ctype <- "Cornfield"
+    cfOR.p <- c(); cfOR.l <- c(); cfOR.u <- c()
+    if(length(dim(dat)) == 3){
+      for(i in 1:dim(dat)[3]){
+        # .tmp <- .funORcfield(dat[,,i], conf.level)
+        cfOR.p <- Oe.p / Oo.p
+        cfOR.l <- NA
+        cfOR.u <- NA
+      }
+    }
+    if(length(dim(dat)) == 2){
+      # .tmp <- .funORcfield(dat, conf.level)
+      cfOR.p <- Oe.p / Oo.p
+      cfOR.l <- NA
+      cfOR.u <- NA
+    }
   }
   
   # Individual strata odds ratio - score confidence limits:
@@ -1044,11 +1065,21 @@
   cwOR.u     <- .tmp[3]
   
   # Crude odds ratio - Cornfield confidence limits:
+  # Only calculate Cornfield confidence limits if N < 500; function very slow with large numbers otherwise:
+  if(sum(total) < 500){
   ccfOR.ctype <- "Cornfield"
   .tmp        <- .funORcfield(apply(dat, MARGIN = c(1,2), FUN = sum), conf.level)
   ccfOR.p     <- .tmp[1]
   ccfOR.l     <- .tmp[2]
   ccfOR.u     <- .tmp[3]
+  }
+  
+  if(sum(total) >= 500){
+    ccfOR.ctype <- "Cornfield"
+    ccfOR.p     <- Oe.p / Oo.p
+    ccfOR.l     <- NA
+    ccfOR.u     <- NA
+  }
   
   # Crude odds ratio - score confidence limits:
   csOR.ctype <- "Score"
@@ -1311,6 +1342,8 @@
     wRR.homog <- sum(wRR. * (lnRR. - lnRR.s.)^2)
     wRR.homog.p <- 1 - pchisq(wRR.homog, df = n.strata - 1)
 
+    wPR.homog <- sum(wRR. * (lnRR. - lnRR.s.)^2)
+    wPR.homog.p <- 1 - pchisq(wPR.homog, df = n.strata - 1)
     
     # Woolf test of homogeneity of odds ratios (Jewell 2004, page 154). First work out the Woolf estimate of the adjusted odds ratio (labelled lnOR.s. here) based on Jewell (2004, page 129):
     lnOR. <- log(((a + 0.5) * (d + 0.5)) / ((b + 0.5) * (c + 0.5)))
@@ -1322,7 +1355,6 @@
     wOR.homog <- sum(wOR. * (lnOR. - lnOR.s.)^2)
     wOR.homog.p <- 1 - pchisq(wOR.homog, df = n.strata - 1)
 
-    
     # Breslow-Day test of homogeneity of odds ratio. Setup calculations. From Jim Robison-Cox, based on Jewell (2004, page 154).
     n11k <- dat[1,1,]
     n21k <- dat[2,1,]
@@ -1399,20 +1431,19 @@
     
     ## Strata odds ratio:
     OR.strata.wald = data.frame(est = wOR.p, lower = wOR.l, upper = wOR.u),
-    OR.strata.score = data.frame(est = scOR.p, lower = scOR.l, upper = scOR.u),
     OR.strata.cfield = data.frame(est = cfOR.p, lower = cfOR.l, upper = cfOR.u),
+    OR.strata.score = data.frame(est = scOR.p, lower = scOR.l, upper = scOR.u),
     OR.strata.mle = data.frame(est = mOR.p, lower = mOR.l, upper = mOR.u),
     
     ## Crude odds ratio:
     OR.crude.wald = data.frame(est = cwOR.p, lower = cwOR.l, upper = cwOR.u),
-    OR.crude.score = data.frame(est = csOR.p, lower = csOR.l, upper = csOR.u),
     OR.crude.cfield = data.frame(est = ccfOR.p, lower = ccfOR.l, upper = ccfOR.u),
+    OR.crude.score = data.frame(est = csOR.p, lower = csOR.l, upper = csOR.u),
     OR.crude.mle = data.frame(est = cmOR.p, lower = cmOR.l, upper = cmOR.u),
-    
+
     ## Mantel-Haenszel odds ratio:
     OR.mh.wald = data.frame(est = sOR.p, lower = sOR.l, upper = sOR.u),
     
-    
     ## Strata attributable risk:
     ARisk.strata.wald = data.frame(est = wARisk.p, lower = wARisk.l, upper = wARisk.u),
     ARisk.strata.score = data.frame(est = scARisk.p, lower = scARisk.l, upper = scARisk.u),
@@ -1525,7 +1556,8 @@
     
     res$wOR.homog = data.frame(test.statistic = wOR.homog, df = n.strata - 1, p.value = wOR.homog.p)
     res$bOR.homog = data.frame(test.statistic = bOR.homog, df = n.strata - 1, p.value = bOR.homog.p)
-    
+
+    res$wPR.homog = data.frame(test.statistic = wPR.homog,  df = n.strata - 1, p.value = wPR.homog.p)    
     res$wRR.homog = data.frame(test.statistic = wRR.homog,  df = n.strata - 1, p.value = wRR.homog.p)
   }    
   
@@ -1543,8 +1575,8 @@
       RR.strata.score    = res$RR.strata.score,
       
       OR.strata.wald     = res$OR.strata.wald,
-      OR.strata.score    = res$OR.strata.score,
       OR.strata.cfield   = res$OR.strata.cfield,
+      OR.strata.score    = res$OR.strata.score,
       OR.strata.mle      = res$OR.strata.mle,
       
       ARisk.strata.wald  = res$ARisk.strata.wald,
@@ -1596,9 +1628,10 @@
       RR.mh.wald         = res$RR.mh.wald,
       
       OR.strata.wald     = res$OR.strata.wald,
-      OR.strata.score    = res$OR.strata.score,
       OR.strata.cfield   = res$OR.strata.cfield,
+      OR.strata.score    = res$OR.strata.score,
       OR.strata.mle      = res$OR.strata.mle,
+      
       OR.crude.wald      = res$OR.crude.wald,
       OR.crude.score     = res$OR.crude.score,
       OR.crude.cfield    = res$OR.crude.cfield,
@@ -1631,7 +1664,7 @@
       RR.homog.woolf  = res$wRR.homog,
       OR.homog.woolf  = res$wOR.homog,
       OR.homog.brday =  res$bOR.homog)
-    
+
     ## Define tab:
     if(outcome == "as.columns"){
       r1 <- c(sa, sb, sN1, cIRiske.p, cOe.p)
@@ -1658,7 +1691,6 @@
     out <- list(method = "cohort.count", n.strata = n.strata, conf.level = conf.level, res = res, massoc = massoc, tab = tab)
   }
   
-  
   ## method = "cohort.time", single strata:
   if(method == "cohort.time" & n.strata == 1){
     
@@ -1752,15 +1784,14 @@
     out <- list(method = "cohort.time", n.strata = n.strata, conf.level = conf.level, res = res, massoc = massoc, tab = tab)
   }
   
-  
   ## method == "case.control", single strata:
   if(method == "case.control" & n.strata == 1){
     
     ## Verbose part:
     massoc <- list(
       OR.strata.wald      = res$OR.strata.wald,
-      OR.strata.score     = res$OR.strata.score,
       OR.strata.cfield    = res$OR.strata.cfield,
+      OR.strata.score     = res$OR.strata.score,
       OR.strata.mle       = res$OR.strata.mle,
       
       ARisk.strata.wald   = res$ARisk.strata.wald,
@@ -1805,23 +1836,23 @@
     ## Verbose part:
     massoc <- list(
       OR.strata.wald      = res$OR.strata.wald,
-      OR.strata.score     = res$OR.strata.score,
       OR.strata.cfield    = res$OR.strata.cfield,
+      OR.strata.score     = res$OR.strata.score,
       OR.strata.mle       = res$OR.strata.mle,
       
-      OR.crude.wald      = res$OR.crude.wald,
-      OR.crude.score     = res$OR.crude.score,
-      OR.crude.cfield    = res$OR.crude.cfield,
-      OR.crude.mle       = res$OR.crude.mle,
-      OR.mh.wald         = res$OR.mh.wald,       
+      OR.crude.wald       = res$OR.crude.wald,
+      OR.crude.cfield     = res$OR.crude.cfield,
+      OR.crude.score      = res$OR.crude.score,
+      OR.crude.mle        = res$OR.crude.mle,
+      OR.mh.wald          = res$OR.mh.wald,       
       
-      ARisk.strata.wald  = res$ARisk.strata.wald,
-      ARisk.strata.score = res$ARisk.strata.score,
-      ARisk.crude.wald   = res$ARisk.crude.wald,
-      ARisk.crude.score  = res$ARisk.crude.score,
-      ARisk.mh.wald      = res$ARisk.mh.wald,
-      ARisk.mh.sato      = res$ARisk.mh.sato,
-      ARisk.mh.green     = res$ARisk.mh.green,
+      ARisk.strata.wald   = res$ARisk.strata.wald,
+      ARisk.strata.score  = res$ARisk.strata.score,
+      ARisk.crude.wald    = res$ARisk.crude.wald,
+      ARisk.crude.score   = res$ARisk.crude.score,
+      ARisk.mh.wald       = res$ARisk.mh.wald,
+      ARisk.mh.sato       = res$ARisk.mh.sato,
+      ARisk.mh.green      = res$ARisk.mh.green,
       
       PARisk.strata.wald  = res$PARisk.strata.wald,
       PARisk.strata.piri  = res$PARisk.strata.piri,
@@ -1834,12 +1865,12 @@
       PAFest.strata.wald  = res$PAFest.strata.wald,
       PAFest.crude.wald   = res$PAFest.crude.wald,
       
-      chisq.strata  = res$chisq.strata,
-      chisq.crude   = res$chisq.crude,
-      chisq.mh      = res$chisq.mh,
+      chisq.strata        = res$chisq.strata,
+      chisq.crude         = res$chisq.crude,
+      chisq.mh            = res$chisq.mh,
       
-      OR.homog.woolf  = res$wOR.homog,
-      OR.homog.brday  = res$bOR.homog)
+      OR.homog.woolf      = res$wOR.homog,
+      OR.homog.brday      = res$bOR.homog)
     
     ## Define tab:
     if(outcome == "as.columns"){
@@ -1867,8 +1898,7 @@
     ## Output creation part:
     out <- list(method = "case.control", n.strata = n.strata, conf.level = conf.level, res = res, massoc = massoc, tab = tab)
   }
-  
-  
+
   ## method == "cross.sectional", single strata:
   if(method == "cross.sectional" & n.strata == 1){
     
@@ -1878,8 +1908,8 @@
       PR.strata.score    = res$RR.strata.score,
       
       OR.strata.wald     = res$OR.strata.wald,
-      OR.strata.score    = res$OR.strata.score,
       OR.strata.cfield   = res$OR.strata.cfield,
+      OR.strata.score    = res$OR.strata.score,
       OR.strata.mle      = res$OR.strata.mle,
       
       ARisk.strata.wald    = res$ARisk.strata.wald,
@@ -1931,12 +1961,12 @@
       PR.mh.wald         = res$RR.mh.wald,
       
       OR.strata.wald     = res$OR.strata.wald,
-      OR.strata.score    = res$OR.strata.score,
       OR.strata.cfield   = res$OR.strata.cfield,
+      OR.strata.score    = res$OR.strata.score,
       OR.strata.mle      = res$OR.strata.mle,
       OR.crude.wald      = res$OR.crude.wald,
-      OR.crude.score     = res$OR.crude.score,
       OR.crude.cfield    = res$OR.crude.cfield,
+      OR.crude.score     = res$OR.crude.score,
       OR.crude.mle       = res$OR.crude.mle,
       OR.mh.wald         = res$OR.mh.wald,
       
@@ -1963,6 +1993,7 @@
       chisq.crude  = res$chisq.crude,
       chisq.mh     = res$chisq.mh,
       
+      PR.homog.woolf  = res$wPR.homog,
       RR.homog.woolf  = res$wRR.homog,
       OR.homog.woolf  = res$wOR.homog,
       OR.homog.brday  = res$bOR.homog)
@@ -2049,15 +2080,17 @@ print.epi.2by2 <- function(x, ...) {
       ))
     })
     cat("\n-------------------------------------------------------------------")
-    p <- ifelse(as.numeric(x$res$chisq.strata)[3] < 0.001, "< 0.001", round(as.numeric(x$res$chisq.strata)[3], digits = 3))
-    
-    cat("\n", "Test that odds ratio = 1: chi2(1) =", as.numeric(round(x$res$chisq.strata[1], digits = 3)), "Pr>chi2 =", p)
-    
+
+    p <-  ifelse(as.numeric(x$res$chisq.strata)[3] < 0.001, "<0.001", sprintf("%.2f", x$res$chisq.strata[3]))
+    pdf <- as.numeric(x$res$chisq.strata[2])
+
+    cat("\n", " Test that OR = 1: chi2(", pdf,   ") = ", sprintf("%.3f", x$res$chisq.strata[1]),  " Pr>chi2 = ", p, sep = "")
     cat("\n", "Wald confidence limits")
     cat("\n", "CI: confidence interval") 
     cat("\n", "*", x$res$count.units, "\n")
   }
   
+  
   ## cohort.count ---  multiple strata
   if(x$method == "cohort.count" & x$n.strata > 1){
     
@@ -2117,27 +2150,24 @@ print.epi.2by2 <- function(x, ...) {
       ))
     })
     cat("\n-------------------------------------------------------------------")
-    wrrp <-  ifelse(as.numeric(x$res$wRR.homog)[3] < 0.001, "<0.001", round(as.numeric(x$res$wRR.homog)[3], digits = 3))
+    wrrp <-  ifelse(as.numeric(x$res$wRR.homog)[3] < 0.001, "<0.001", sprintf("%.2f", x$res$wPR.homog[3]))
     wrrdf <- as.numeric(x$res$wRR.homog[2])
     
-    worp <-  ifelse(as.numeric(x$res$wOR.homog)[3] < 0.001, "<0.001", round(as.numeric(x$res$wOR.homog)[3], digits = 3))
+    worp <-  ifelse(as.numeric(x$res$wOR.homog)[3] < 0.001, "<0.001", sprintf("%.2f", x$res$wOR.homog[3]))
     wordf <- as.numeric(x$res$wOR.homog[2])
-
-    mhp <-   ifelse(as.numeric(x$res$chisq.mh)[3] < 0.001, "<0.001", round(as.numeric(x$res$chisq.mh)[3], digits = 3))
-    mhdf <-  as.numeric(x$res$chisq.mh[2])
-        
-    cat("\n", " M-H test of homogeneity of incidence risk ratios: chi2(", wrrdf, ") = ", as.numeric(round(x$res$wRR.homog[1], digits = 3)), " Pr>chi2 = ", wrrp, sep = "")
-
-    cat("\n", " M-H test of homogeneity of odds ratios: chi2(", wordf, ") = ", as.numeric(round(x$res$wOR.homog[1], digits = 3)), " Pr>chi2 = ", worp, sep = "")
-        
-    cat("\n", " Test that M-H adjusted odds ratio = 1: chi2(", mhdf, ") = ", as.numeric(round(x$res$chisq.mh[1], digits = 3)), " Pr>chi2 = ", mhp, sep = "")
     
+    mhp <-  ifelse(as.numeric(x$res$chisq.mh)[3] < 0.001, "<0.001", sprintf("%.2f", x$res$chisq.mh[3]))
+    mhdf <-  as.numeric(x$res$chisq.mh[2])
+    
+    cat("\n", " M-H test of homogeneity of RRs: chi2(", wrrdf, ") = ", sprintf("%.3f", x$res$wRR.homog[1]), " Pr>chi2 = ", wrrp, sep = "")
+    cat("\n", " M-H test of homogeneity of ORs: chi2(", wordf, ") = ", sprintf("%.3f", x$res$wOR.homog[1]), " Pr>chi2 = ", worp, sep = "")
+    cat("\n", " Test that M-H adjusted OR = 1:  chi2(", mhdf, ") = ",  sprintf("%.3f", x$res$chisq.mh[1]),  " Pr>chi2 = ", mhp, sep = "")
     cat("\n", "Wald confidence limits")
     cat("\n", "M-H: Mantel-Haenszel; CI: confidence interval")        
     cat("\n", "*", x$res$count.units, "\n")
   }
-  
-  
+
+    
   ## cohort.time --- single strata
   if(x$method == "cohort.time" & x$n.strata == 1){
     
@@ -2179,9 +2209,8 @@ print.epi.2by2 <- function(x, ...) {
       
     })
     cat("\n-------------------------------------------------------------------")
-    # p <- ifelse(as.numeric(x$res$chisq.strata)[3] < 0.001, "< 0.001", round(as.numeric(x$res$chisq.strata)[3], digits = 3))
-    
-    # cat("\n", "Test that incidence rate ratio = 1: chi2(1) =", as.numeric(round(x$res$chisq.strata[1], digits = 3)), "Pr>chi2 =", p)
+    # p <-  ifelse(as.numeric(x$res$chisq.strata)[3] < 0.001, "<0.001", sprintf("%.2f", x$res$chisq.strata[3]))
+    # pdf <- as.numeric(x$res$chisq.strata[2])
     
     cat("\n", "Wald confidence limits")
     cat("\n", "CI: confidence interval")
@@ -2230,11 +2259,14 @@ print.epi.2by2 <- function(x, ...) {
       ))
     })
     cat("\n-------------------------------------------------------------------")
-    # rrp <- ifelse(as.numeric(x$res$RR.homog)[3] < 0.001, "< 0.001", round(as.numeric(x$res$RR.homog)[3], digits = 3))
-    # cat("\n", "Test of homogeneity of IRR: X2 test statistic:", as.numeric(round(x$res$RR.homog[1], digits = 3)), "p-value:", rrp)
+    # rrp <-  ifelse(as.numeric(x$res$RR.homog)[3] < 0.001, "<0.001", sprintf("%.2f", x$res$RR.homog[3]))
+    # rrdf <- as.numeric(x$res$RR.homog[2])
     
-    # orp <- ifelse(as.numeric(x$res$OR.homog)[3] < 0.001, "< 0.001", round(as.numeric(x$res$OR.homog)[3], digits = 3))
-    # cat("\n", "Test of homogeneity of  OR: X2 test statistic:", as.numeric(round(x$res$OR.homog[1], digits = 3)), "p-value:", orp)
+    # mhp <-  ifelse(as.numeric(x$res$chisq.mh)[3] < 0.001, "<0.001", sprintf("%.2f", x$res$chisq.mh[3]))
+    # mhdf <-  as.numeric(x$res$chisq.mh[2])
+    
+    # cat("\n", " M-H test of homogeneity of IRRs: chi2(", wrrdf, ") = ", sprintf("%.3f", x$res$wRR.homog[1]), " Pr>chi2 = ", wrrp, sep = "")
+    # cat("\n", " Test that M-H adjusted OR = 1:  chi2(", mhdf, ") = ",  sprintf("%.3f", x$res$chisq.mh[1]),  " Pr>chi2 = ", mhp, sep = "")
     cat("\n", "Wald confidence limits")
     cat("\n", "M-H: Mantel-Haenszel; CI: confidence interval")
     cat("\n", "*", x$res$time.units, "\n")
@@ -2281,10 +2313,10 @@ print.epi.2by2 <- function(x, ...) {
       ))
     })
     cat("\n-------------------------------------------------------------------")
-    p <- ifelse(as.numeric(x$res$chisq.strata)[3] < 0.001, "< 0.001", round(as.numeric(x$res$chisq.strata)[3], digits = 3))
-    
-    cat("\n", "Test that odds ratio = 1: chi2(1) =", as.numeric(round(x$res$chisq.strata[1], digits = 3)), "Pr>chi2 =", p)
+    p <-  ifelse(as.numeric(x$res$chisq.strata)[3] < 0.001, "<0.001", sprintf("%.2f", x$res$chisq.strata[3]))
+    pdf <- as.numeric(x$res$chisq.strata[2])
     
+    cat("\n", " Test that OR = 1: chi2(", pdf,   ") = ", sprintf("%.3f", x$res$chisq.strata[1]),  " Pr>chi2 = ", p, sep = "")
     cat("\n", "Wald confidence limits")
     cat("\n", "CI: confidence interval")
     cat("\n", "*", x$res$count.units, "\n")
@@ -2333,16 +2365,14 @@ print.epi.2by2 <- function(x, ...) {
       ))
     })
     cat("\n-------------------------------------------------------------------")
-    worp <-  ifelse(as.numeric(x$res$wOR.homog)[3] < 0.001, "<0.001", round(as.numeric(x$res$wOR.homog)[3], digits = 3))
+    worp <-  ifelse(as.numeric(x$res$wOR.homog)[3] < 0.001, "<0.001", sprintf("%.2f", x$res$wOR.homog[3]))
     wordf <- as.numeric(x$res$wOR.homog[2])
     
-    mhp <-   ifelse(as.numeric(x$res$chisq.mh)[3] < 0.001, "<0.001", round(as.numeric(x$res$chisq.mh)[3], digits = 3))
+    mhp <-  ifelse(as.numeric(x$res$chisq.mh)[3] < 0.001, "<0.001", sprintf("%.2f", x$res$chisq.mh[3]))
     mhdf <-  as.numeric(x$res$chisq.mh[2])
     
-    cat("\n", " M-H test of homogeneity of odds ratios: chi2(", wordf, ") = ", as.numeric(round(x$res$wOR.homog[1], digits = 3)), " Pr>chi2 = ", worp, sep = "")
-    
-    cat("\n", " Test that M-H adjusted odds ratio = 1: chi2(", mhdf, ") = ", as.numeric(round(x$res$chisq.mh[1], digits = 3)), " Pr>chi2 = ", mhp, sep = "")
-    
+    cat("\n", " M-H test of homogeneity of ORs: chi2(", wordf, ") = ", sprintf("%.3f", x$res$wOR.homog[1]), " Pr>chi2 = ", worp, sep = "")
+    cat("\n", " Test that M-H adjusted OR = 1:  chi2(", mhdf, ") = ",  sprintf("%.3f", x$res$chisq.mh[1]),  " Pr>chi2 = ", mhp, sep = "")
     cat("\n", "Wald confidence limits")
     cat("\n", "M-H: Mantel-Haenszel; CI: confidence interval") 
     cat("\n", "*", x$res$count.units, "\n")
@@ -2395,10 +2425,10 @@ print.epi.2by2 <- function(x, ...) {
       ))
     })
     cat("\n-------------------------------------------------------------------")
-    p <- ifelse(as.numeric(x$res$chisq.strata)[3] < 0.001, "< 0.001", round(as.numeric(x$res$chisq.strata)[3], digits = 3))
-
-    cat("\n", "Test that odds ratio = 1: chi2(1) =", as.numeric(round(x$res$chisq.strata[1], digits = 3)), "Pr>chi2 =", p)
+    p <-  ifelse(as.numeric(x$res$chisq.strata)[3] < 0.001, "<0.001", sprintf("%.2f", x$res$chisq.strata[3]))
+    pdf <- as.numeric(x$res$chisq.strata[2])
     
+    cat("\n", " Test that OR = 1: chi2(", pdf,   ") = ", sprintf("%.3f", x$res$chisq.strata[1]),  " Pr>chi2 = ", p, sep = "")
     cat("\n", "Wald confidence limits")
     cat("\n", "CI: confidence interval")
     cat("\n", "*", x$res$count.units, "\n")
@@ -2463,21 +2493,18 @@ print.epi.2by2 <- function(x, ...) {
       ))
     })
     cat("\n-------------------------------------------------------------------")
-    wrrp <-  ifelse(as.numeric(x$res$wRR.homog)[3] < 0.001, "<0.001", round(as.numeric(x$res$wRR.homog)[3], digits = 3))
+    wrrp <-  ifelse(as.numeric(x$res$wRR.homog)[3] < 0.001, "<0.001", sprintf("%.2f", x$res$wPR.homog[3]))
     wrrdf <- as.numeric(x$res$wRR.homog[2])
-    
-    worp <-  ifelse(as.numeric(x$res$wOR.homog)[3] < 0.001, "<0.001", round(as.numeric(x$res$wOR.homog)[3], digits = 3))
+
+    worp <-  ifelse(as.numeric(x$res$wOR.homog)[3] < 0.001, "<0.001", sprintf("%.2f", x$res$wOR.homog[3]))
     wordf <- as.numeric(x$res$wOR.homog[2])
-    
-    mhp <-   ifelse(as.numeric(x$res$chisq.mh)[3] < 0.001, "<0.001", round(as.numeric(x$res$chisq.mh)[3], digits = 3))
+
+    mhp <-  ifelse(as.numeric(x$res$chisq.mh)[3] < 0.001, "<0.001", sprintf("%.2f", x$res$chisq.mh[3]))
     mhdf <-  as.numeric(x$res$chisq.mh[2])
-    
-    cat("\n", " M-H test of homogeneity of prevalence ratios: chi2(", wrrdf, ") = ", as.numeric(round(x$res$wRR.homog[1], digits = 3)), " Pr>chi2 = ", wrrp, sep = "")
-    
-    cat("\n", " M-H test of homogeneity of odds ratios: chi2(", wordf, ") = ", as.numeric(round(x$res$wOR.homog[1], digits = 3)), " Pr>chi2 = ", worp, sep = "")
-    
-    cat("\n", " Test that M-H adjusted odds ratio = 1: chi2(", mhdf, ") = ", as.numeric(round(x$res$chisq.mh[1], digits = 3)), " Pr>chi2 = ", mhp, sep = "")
-    
+
+    cat("\n", " M-H test of homogeneity of PRs: chi2(", wrrdf, ") = ", sprintf("%.3f", x$res$wPR.homog[1]), " Pr>chi2 = ", wrrp, sep = "")
+    cat("\n", " M-H test of homogeneity of ORs: chi2(", wordf, ") = ", sprintf("%.3f", x$res$wOR.homog[1]), " Pr>chi2 = ", worp, sep = "")
+    cat("\n", " Test that M-H adjusted OR = 1:  chi2(", mhdf, ") = ",  sprintf("%.3f", x$res$chisq.mh[1]),  " Pr>chi2 = ", mhp, sep = "")
     cat("\n", "Wald confidence limits")
     cat("\n", "M-H: Mantel-Haenszel; CI: confidence interval") 
     cat("\n", "*", x$res$count.units, "\n")
